//
//  MIDI_Model.m
//  MIDI Tool
//
//  Created by John Geelen on 23-05-11.
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program MIDI Echo and PlaySoftMIDI.
//	Nanosecond Timing in OS X used in printPacketInfo: http://developer.apple.com/qa/qa2004/qa1398.html
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	Cocoa, OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "MIDI_Model.h"

	#pragma mark -
	#pragma mark === Static Methods ===
//	*********************************************************************************************************************************
static void myMIDINotifyProc(const MIDINotification *message, void *refCon);
static void	MyReadProc(const MIDIPacketList *pktlist, void *refCon, void *connRefCon);


@implementation MIDI_Model

	#pragma mark -
	#pragma mark === Accessor Methods ===
//	*********************************************************************************************************************************
	@synthesize channelIn, channelOut;

- (id)init
{
	if(![super init])
		return nil;
	
	channelIn = 0;
	channelOut = 0;
	
	return self;
}

- (void)dealloc
{
	
	[super dealloc];
}

	#pragma mark -
	#pragma mark === MIDI Methods ===
//	*********************************************************************************************************************************
- (void)mmMidiSetUp
{
	OSStatus			status;
	MIDIClientRef		client = nulDev;
	
// create client and ports
	
	NSLog(@"setChannelIn = %i", channelIn);
	NSLog(@"setChannelOut = %i", channelOut);
	
	status = MIDIClientCreate(CFSTR("MIDI Echo"), myMIDINotifyProc, NULL, &client);
	if(status != noErr){
		NSLog(@"MIDIClientCreate failed");
	}
	status = MIDIInputPortCreate(client, CFSTR("Input"), MyReadProc, self, &PortIn);
	if(status != noErr){
		NSLog(@"MIDIInputPortCreate failed");
	}
	status = MIDIOutputPortCreate(client, CFSTR("Output"), &PortOut);
	if(status != noErr){
		NSLog(@"MIDIOutputPortCreate failed");
	}
	
}

- (void)SetMidiChannelIn:(Byte)channel
{
	[self setChannelIn:channel];
	NSLog(@"setChannelIn = %i", channelIn);
}

- (void)SetMidiChannelOut:(Byte)channel
{
	[self setChannelOut:channel];
	NSLog(@"setChannelOut = %i", channelOut);
}

- (void)mmOpenConnectionFromSource:(MIDIEndpointRef)source
{
	Source = source;
	MIDIPortConnectSource(PortIn, Source, NULL);
}

- (void)mmCloseConnectionFromSource:(MIDIEndpointRef)source
{
	Source = source;
	MIDIPortDisconnectSource(PortIn, Source);
}

- (BOOL)mmSendMidi:(MIDIEndpointRef)destination packetList:(MIDIPacketList *)packetlist
{
	Destination = destination;
	OSStatus status = MIDISend(PortOut, Destination, packetlist);
	if(status != noErr){
		NSLog(@"problem to send MIDI data");
		return NO;
	} else {
		return YES;
	}
}

- (void)mmOpenConnectionsFromAllSources
{
	unsigned long		n, i;
	
	n = MIDIGetNumberOfSources();
	NSLog(@"sources: %lu",n);
	if(n > 0){
		for (i = 0; i < n; ++i) {
			MIDIEndpointRef src = MIDIGetSource(i);
			MIDIPortConnectSource(PortIn, src, NULL);
		}
	}
}

- (void)mmCloseConnectionsFromAllSources
{
	unsigned long		n, i;
	
	n = MIDIGetNumberOfSources();
		//	NSLog(@"sources: %lu",n);
	if(n > 0){
		for (i = 0; i < n; ++i) {
			MIDIEndpointRef src = MIDIGetSource(i);
			MIDIPortDisconnectSource(PortIn, src);
		}
//		[midiDevInPopUp selectItem:[midiDevInPopUp lastItem]];
	}
}

//	void printPacketInfo(const MIDIPacket* packet)
- (void)printPacketInfo:(const MIDIPacket*) packet
{
//	Nanosecond Timing in OS X used in printPacketInfo: http://developer.apple.com/qa/qa2004/qa1398.html
	double timeinsec = 0.0;
	mach_timebase_info_data_t info;
	if (mach_timebase_info(&info) == 0) {
		timeinsec = mach_absolute_time()*1e-9*(double)info.numer/info.denom;
	}
	
	double stampinsec = packet->timeStamp / (double)1e9;
//	printf("%9.3lfs\t-\t", timeinsec);
	NSLog(@"Time in sec %9.3lfs", timeinsec);
//	printf("%9.3lfs\t", stampinsec);
	NSLog(@"Stamp in sec %9.3lfs", stampinsec);
// display different in times in microseconds
//	printf("Time in - Stamp %9.0lfus\t", (timeinsec-stampinsec)*1000000);
	NSLog(@"Time in - Stamp %9.0lfus", (timeinsec-stampinsec)*1000000); 
	int i;
	for (i=0; i<packet->length; i++) {
		if (packet->data[i] < 0x7f) {
		//	printf("%d ", packet->data[i]);
			NSLog(@"%d ", packet->data[i]);
		} else {
		//	printf("0x%x ", packet->data[i]);
			NSLog(@"0x%x ", packet->data[i]);
		}
	}
//	printf("\n");
	[[NSNotificationCenter defaultCenter] postNotificationName:@"MIDIMessage" object:nil];
}

	#pragma mark -
	#pragma mark === Callback Methods ===
//	*********************************************************************************************************************************
static void myMIDINotifyProc(const MIDINotification *message, void *refCon)
{
	if (message->messageID == kMIDIMsgSetupChanged) {
	//	local autorelease pool because this is a secondary thread and we call a cocoa method from this thread
		NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];  
		[[NSNotificationCenter defaultCenter] postNotificationName:@"MIDISetupChanged" object:nil];
		[pool release];
	}    
}

static void	MyReadProc(const MIDIPacketList *packetList, void *refCon, void *connRefCon)
{
//	MIDI receive call back
	NSLog(@"\nMyReadProc");
	MIDI_Model *		midiModelclassPtr = (MIDI_Model *)refCon;
	MIDIPacket *		packet = (MIDIPacket*)packetList->packet;
	int					j;
	int					count = packetList->numPackets;
	
//	local autorelease pool because this is a secondary thread and we call cocoa methods from this thread
	NSAutoreleasePool *	pool = [[NSAutoreleasePool alloc] init];
	
	for (j=0; j<count; j++) {
		[midiModelclassPtr printPacketInfo:packet];		//	here we call a cocoa method
	//	printPacketInfo(packet);						//	here we call a C function
		packet = MIDIPacketNext(packet);
	}
	[pool release];
}

	//	MIDI echo not implemented. In MIDI Echo received MIDI data is transmitted through MIDI out port.
/*
 code from Apple's example MIDI Echo
 if (gOutPort != nulDev && gDest != nulDev) {
 MIDIPacket *packet = (MIDIPacket *)pktlist->packet;
 for (unsigned int j = 0; j < pktlist->numPackets; ++j) {
 for (int i = 0; i < packet->length; ++i) {
 //				printf("%02X ", packet->data[i]);
 
 //	rechannelize status bytes
 if (packet->data[i] >= 0x80 && packet->data[i] < 0xF0)
 packet->data[i] = (packet->data[i] & 0xF0) | gOutChannel;
 }
 
 //			printf("\n");
 packet = MIDIPacketNext(packet);
 }
 
 MIDISend(gOutPort, gDest, pktlist);
 }
 */

@end
