//
//  MIDI_PLayer.m
//  MIDI Tool
//
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program MIDI Echo and PlaySoftMIDI.
//	Comments from Apple:
//	This is a simple test case of making an simple graph with a DLSSynth, Limiter and Ouput unit.
//	We're going to use a graph because its easier for it to just handle the setup and connections, etc...
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	Cocoa, OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "MIDI_PLayer.h"

@implementation MIDI_PLayer

//	This call creates the Graph and the Synth unit...
- (OSStatus)mpCreateAUGraph
{
	OSStatus					result;
	AUNode						synthNode, limiterNode, outNode;
	UInt8						midiChannelInUse = 0;
	AudioComponentDescription	cd;
	
//	create AUGraph
	require_noerr (result = NewAUGraph(&graph), home);

//	add node audio unit musicdive to audio graph
	cd.componentManufacturer = kAudioUnitManufacturer_Apple;
	cd.componentType = kAudioUnitType_MusicDevice;
	cd.componentSubType = kAudioUnitSubType_DLSSynth;
	cd.componentFlags = 0;
	cd.componentFlagsMask = 0;
	require_noerr (result = AUGraphAddNode(graph, &cd, &synthNode), home);	

//	add node audio unit peak limiter to audio graph
	cd.componentType = kAudioUnitType_Effect;
	cd.componentSubType = kAudioUnitSubType_PeakLimiter;  
	result = AUGraphAddNode (graph, &cd, &limiterNode);
	
//	add node audio unit default audio unit to audio graph
	cd.componentType = kAudioUnitType_Output;
	cd.componentSubType = kAudioUnitSubType_DefaultOutput;  
	require_noerr (result = AUGraphAddNode (graph, &cd, &outNode), home);
	
//	open audio graph
	require_noerr (result = AUGraphOpen (graph), home);

//	connect nodes in graph
	require_noerr (result = AUGraphConnectNodeInput (graph, synthNode, 0, limiterNode, 0), home);
	require_noerr (result = AUGraphConnectNodeInput (graph, limiterNode, 0, outNode, 0), home);
	
//	get the Synth Unit...
	require_noerr (result = AUGraphNodeInfo(graph, synthNode, 0, &synthUnit), home);
	
//	initialize the graph
	require_noerr (result = AUGraphInitialize (graph), home);
	
//	set our bank
	require_noerr (result = MusicDeviceMIDIEvent(synthUnit, 
												 kMidiMessage_ControlChange << 4 | midiChannelInUse, 
												 kMidiMessage_BankMSBControl, 0,
												 0/*sample offset*/), home);
	
	require_noerr (result = MusicDeviceMIDIEvent(synthUnit, 
												 kMidiMessage_ProgramChange << 4 | midiChannelInUse, 
												 0/*prog change num*/, 0,
												 0/*sample offset*/), home);
	
	CAShow (graph);		// prints out the internal state of an object to NSLog

home:
	return result;
}


- (OSStatus)mpStartGraph
{
	OSStatus		result;
//	create the audio graph
	require_noerr (result = [self mpCreateAUGraph], home);
//	start the graph
	require_noerr (result = AUGraphStart (graph), home);
	
home:
	return result;
}


- (void)mpStopGraph
{
	if (graph) {
		AUGraphStop (graph);	// stop playback - DisposeAUGraph will do AUGraphStop as well
		DisposeAUGraph (graph);
	}
}


- (OSStatus)mpPlayMidiNote:(UInt8)note velocity:(UInt8)velocity channel:(UInt8)channel
{
	OSStatus	result;
	UInt8		noteOnCommand = kMidiMessage_NoteOn << 4 | channel;

//	printf ("Playing Note: Status: 0x%u, Note: %u, Velocty: %u\n", noteOnCommand, note, velocity);
	require_noerr (result = MusicDeviceMIDIEvent(synthUnit, noteOnCommand, note, velocity, 0), home);

home:
	return result;
}

@end
